import Taro, { Component, Config } from '@tarojs/taro'
import { Provider } from '@tarojs/redux'

import Index from './pages/index'

import configStore from './store'

import './app.scss'

// 如果需要在 h5 环境中开启 React Devtools
// 取消以下注释：
// if (process.env.NODE_ENV !== 'production' && process.env.TARO_ENV === 'h5')  {
//   require('nerv-devtools')
// }

const store = configStore()

class App extends Component {

  /**
   * 指定config的类型声明为: Taro.Config
   *
   * 由于 typescript 对于 object 类型推导只能推出 Key 的基本类型
   * 对于像 navigationBarTextStyle: 'black' 这样的推导出的类型是 string
   * 提示和声明 navigationBarTextStyle: 'black' | 'white' 类型冲突, 需要显示声明类型
   */
  config: Config = {
    pages: [
      'pages/index/index',
      'pages/rank/rank',
      'pages/my/my',
      'pages/enlist/enlist',
      'pages/upload/upload',
      'pages/record/record',
      'pages/army/army',
    ],
    tabBar: {
      color: "#333333",
      borderStyle: 'black',
      selectedColor: "#FF6300",
      backgroundColor: "#ffffff",
      list: [
        {
          text: "首页",
          pagePath: "pages/index/index",
          iconPath: "./static/tabbar/index.png",
          selectedIconPath: "./static/tabbar/index-act.png"
        },
        {
          text: "排行",
          pagePath: "pages/rank/rank",
          iconPath: "./static/tabbar/rank.png",
          selectedIconPath: "./static/tabbar/rank-act.png",
        },
        {
          text: "我的",
          pagePath: "pages/my/my",
          iconPath: "./static/tabbar/my.png",
          selectedIconPath: "./static/tabbar/my-act.png",
        }
      ]
    },
    window: {
      backgroundTextStyle: 'dark',
      navigationBarBackgroundColor: '#fff',
      navigationBarTitleText: 'WeChat',
      navigationBarTextStyle: 'black'
    }
  }

  componentDidMount () {
    (Taro as any).loading = function (title: string) {
      Taro.showLoading({ title: title || '加载中...', mask: true })
    };
    (Taro as any).toast = function(title: string) {
      Taro.showToast({ title, icon: 'none' })
    };
    (Taro as any).$req = function (request, fun) {
      (Taro as any).loading();
      request.then(({ data : { code, message, data }}) => {
        if (code === 200) {
          fun(data)
        } else {
          (Taro as any).toast(message)
        }
      }).catch(() => Taro.hideLoading())
    }
    Taro.hideTabBar()
  }

  componentDidShow () {}

  componentDidHide () {}

  componentDidCatchError () {}

  // 在 App 类中的 render() 函数没有实际作用
  // 请勿修改此函数
  render () {
    return (
      <Provider store={store}>
        <Index />
      </Provider>
    )
  }
}

Taro.render(<App />, document.getElementById('app'))
