import { ComponentClass } from 'react'
import Taro, { Component, Config } from '@tarojs/taro'
import { View, Image, Text, Input, Button } from '@tarojs/components'

import { connect } from '@tarojs/redux'
import { getUserInfo } from '../../actions/userinfo'

import { _api_login } from '../../apis/login'

import './army.scss'

import img_army from '../../static/army.jpg'

import { _api_getArmy, _api_postArmy } from '../../apis/army'

type PageStateProps = {
  userInfo: {
    wx: {
      nickName: string
    }
  }
}
type PageDispatchProps = {
  getUserInfo: (e) => void
}

type PageOwnProps = {}
type PageState = {}
type IProps = PageStateProps & PageDispatchProps & PageOwnProps
interface Index {
  props: IProps;
}

@connect(({ userInfo }) => ({
  userInfo
}), (dispatch) => ({
  getUserInfo (data) {
    dispatch(getUserInfo(data))
  }
}))
class Index extends Component<any, any> {
  config: Config = {
    navigationBarTitleText: '军令状',
  }

  constructor(props) {
    super(props)
    this.state = {
      img: '',
      num: 0,
      money: 0,
      status: 0,
      promise: false
    }
  }

  onShareAppMessage () {
    return {
      title: '我的军令状',
      path: `/pages/army/army?img=${this.state.img}`
    }
  }
  
  async componentDidMount() {
    if (this.$router.params.img) { // 从分享页面来的
      this.setState({
        status: 0,
        img: this.$router.params.img
      })
    } else {
      this.request()
    }
  }

  componentDidShow () { }

  componentDidHide () { }

  render () {
    return (
      <View className='army'>
        <Image src={this.state.img ? this.state.img : img_army} className="armyImg" />
        {
          this.state.status === 1 && <View className="promised">
            <Button openType="share" className="share at-icon at-icon-share"></Button>
            <Button className="shareText">分享</Button>
          </View>
        }
        {
          this.state.status === -1 && <View className={'willPromise' + ' ' + (this.state.promise ? 'ac' : '')}>
            {
              this.state.promise && <View className="promiseInput">
                <View className="inputItem title">输入数量</View>
                <View className="inputItem">
                  <Text>卖货数量：</Text>
                  <Input className="inputMoney" type='number' focus={true} placeholder="请输入卖货数量" onInput={({ detail : { value: num }}) => this.setState({ num })} />
                </View>
                <View className="inputItem">
                  <Text>红包金额：</Text>
                  <Input className="inputMoney" type='digit' placeholder="请输入红包金额" onInput={({ detail : { value: money }}) => this.setState({ money })} />
                </View>
                <View className="inputItem submit" onClick={this.submit}>提交</View>
              </View>
            }
            <View className="promise" onClick={() => this.setState({ promise: true })}>签署军令状</View>
          </View>
        }
      </View>
    )
  }

  request() { // 请求发起请求
    (Taro as any).loading()
    _api_getArmy({}).then(({ data: { data }}) => {
      this.setState({ ...data, status: data.img ? 1 : -1 })
      Taro.hideLoading()
    })
  }

  save() { // 保存图片
    Taro.getImageInfo({ src: this.state.img }).then(({ path: filePath }) => {
      Taro.saveImageToPhotosAlbum({ filePath }).catch(() => {
        Taro.showModal({
          title: '提示',
          content: '是否允许大卫博士实战营保存图片到您的相册？',
          success: ({ confirm }) => {
            if (confirm) {
              Taro.openSetting({
                success: ({ authSetting }) => {
                  if (authSetting['scope.writePhotosAlbum']) {
                    this.save()
                  }
                }
              })
            }
          }
        })
      })
    })
  }

  submit() {
    const num = Number(this.state.num)
    const money = Number(this.state.money)
    console.log(num, money)
    if (num && money) {
      Taro.showModal({
        title: '军令状',
        content: `保证在21天里，每周至少零售 ${num} 套产品；如果做不到，自愿发 ${money} 红包`,
        success: ({ confirm }) => {
          if (confirm) {
            (Taro as any).loading();
            _api_postArmy({ promise_num: num, gift_money: money }).then(() => {
              Taro.hideLoading()
              this.request()
            })
          }
        }
      })
    } else {
      (Taro as any).toast('请输入正确的卖货数量和红包金额')
    }
  }

}

export default Index as ComponentClass<PageOwnProps, PageState>
