import { ComponentClass } from 'react'
import Taro, { Component, Config } from '@tarojs/taro'
import { View, Button, Text, OpenData, Image } from '@tarojs/components'

import { connect } from '@tarojs/redux'
import { getUserInfo } from '../../actions/userinfo'

import { _api_getInfo } from '../../apis/login'

import './enlist.scss'

import index_banner from '../../static/index_banner.png'

import { _api_signup, _api_payStatus, _api_getURL } from '../../apis/signup'

type PageStateProps = {
  userInfo: {
    level: string,
    phone: string,
    nickname: string,
    sign_up: boolean,
    money: number,
    start_time: string,
    end_time: string,
    live_start_time: string,
    live_end_time: string,
    link: string,
    link_img: string,
    code: string
  }
}
type PageDispatchProps = {
  getUserInfo: (e) => void
}

type PageOwnProps = {}
type PageState = {}
type IProps = PageStateProps & PageDispatchProps & PageOwnProps
interface Index {
  props: IProps;
}

@connect(({ userInfo, _api_payStatus }) => ({
  userInfo
}), (dispatch) => ({
  getUserInfo (data) {
    dispatch(getUserInfo(data))
  }
}))
class Index extends Component<any, any> {
  config: Config = {
    navigationBarTitleText: '报名缴费'
  }
  requesting: boolean

  constructor(props) {
    super(props)
  }
  
  async componentDidMount() { }

  componentDidShow () { }

  componentDidHide () { }

  render () {
    return (
      <View className='enlist'>
        <View className="time">{this.getDate(this.props.userInfo.start_time)}-{this.getDate(this.props.userInfo.end_time)}</View>
        <Image className="indexBanner" src={index_banner} />
        <View className="title">报名人信息</View>
        <View className="userInfo">
          <View className="userInfoItem">
            <Text className="left">姓名</Text>
            <Text>{this.props.userInfo.nickname}</Text>
          </View>
          <Text className="line" />
          <View className="userInfoItem">
            <Text className="left">手机号</Text>
            <Text>{this.props.userInfo.phone}</Text>
          </View>
          <Text className="line" />
          <View className="userInfoItem">
            <Text className="left">代理等级</Text>
            <Text>{this.props.userInfo.level}</Text>
          </View>
        </View>
        {
          this.props.userInfo.sign_up ? 
          <View className="live">
            <View className="liveTime"><Text className="baseColor">直播时间：</Text>{this.getDate(this.props.userInfo.live_start_time)} - {this.getDate(this.props.userInfo.live_end_time)}</View>
            <View className="liveInfo">
              <View className="left">
                <Image onClick={this.preview} src={this.props.userInfo.link_img} />
                <Text className="baseColor">直播间二维码</Text>
              </View>
              <View className="right">
                <Text className="baseColor">我的直播间密码</Text>
                <Text className="code">{this.props.userInfo.code}</Text>
                <Button className="copy baseColor" open-type='contact' onClick={this.copy}>复制</Button>
              </View>
            </View>
          </View> :
          <View className="readme">
            <View className="readmeTitle baseColor">报名须知</View>
            <View className="item item1">报名之后非特殊原因无法退回</View>
            <View className="item item2">观看直播二维码及房间密码会在报名成功后发放</View>
            <View className="item item3">参赛需缴纳 <Text className="baseColor">￥{this.props.userInfo.money}</Text> 报名费</View>
          </View>
        }
        {
          !this.props.userInfo.sign_up &&
          <View className="sign" onClick={this.enlist}>￥{this.props.userInfo.money}报名</View>
        }
      </View>
    )
  }

  getDate(dataTime: string) {
    return dataTime ? dataTime.split(' ')[0] : ''
  } 

  enlist() {
    (Taro as any).loading();
    if (!this.requesting) {
      this.requesting = true;
      (Taro as any).$req(_api_signup({}), data => {
        Taro.requestPayment({
          ...data,
          success: () => {
            (Taro as any).toast('支付成功')
          },
          complete: async () => {
            const { data: { code }} = await _api_payStatus({})
            if (code === 200) {
              this.props.getUserInfo((await _api_getInfo({})).data.data)
              Taro.showTabBar(); // 显示 tabbar
              this.requesting = false
              Taro.hideLoading()
            } else {
              Taro.navigateBack()
            }
          }
        })
      })
    }
  }

  preview() {
    Taro.previewImage({ urls: [this.props.userInfo.link_img] })
  }

  copy() {
    setTimeout(() => _api_getURL({}), 1000)
    Taro.setClipboardData({ data: this.props.userInfo.code })
  }

}

export default Index as ComponentClass<PageOwnProps, PageState>
