import Taro, { Component } from '@tarojs/taro'
import { View, Button, Text, Input, OpenData } from '@tarojs/components'

import './BindPhone.scss'

import { _api_getCode, _api_bindPhone } from '../../../apis/login'

import { connect } from '@tarojs/redux'
import { getUserInfo } from '../../../actions/userinfo'


@connect(({ userInfo }) => ({
  userInfo
  }), (dispatch) => ({
  getUserInfo (data) {
      dispatch(getUserInfo(data))
  }
}))
class BindPhone extends Component<any, any> {

  constructor(props) {
    super(props)
    this.state = {
      showGetUserInfoButton: false,
      phone: '',
      code: '',
      count: 0
    }
  }

  componentDidMount() {
    console.log(this.props)
  }

  render () {
    return (
      <View className='BindPhone'>
          <View className="content">
            <View className="header">
              <OpenData className="avatar" type='userAvatarUrl'/>
            </View>
            <View className="form">
                <View className="formItem">
                    <Input type='number' focus placeholder='请输入手机号' maxLength={11} onInput={({ detail: { value: phone }}) => this.setState({ phone })}/>
                </View>
                <View className="formItem">
                    <Input className="code" type='number' placeholder='输入6位验证码' maxLength={6} onInput={({ detail: { value: code }}) => this.setState({ code })} />
                    <View className="getCode" onClick={this.getCode}>获取验证码 {
                        this.state.count && <Text>{ this.state.count }</Text>
                    }</View>
                </View>
            </View>
            <Button className="submit" onClick={this.submit}>立即绑定</Button>
          </View>
      </View>
    )
  }

  getCode() {
      if (this.state.count) {
        (Taro as any).toast('验证码已发送')
      } else {
        if (this.state.phone.length > 4) {
            (Taro as any).loading()
            _api_getCode({ phone: this.state.phone }).then(({ data: { code, message }}) => {
                Taro.hideLoading()
                if (code === 200) {
                    this.setState({ count: 120 });
                    (this as any).timer = setInterval(() => {
                        this.setState({ count: this.state.count - 1 })
                        if (this.state.count === 1) {
                            clearInterval((this as any).timer)
                        }
                    }, 1000)
                } else {
                    (Taro as any).toast(message)
                }
            })
        } else {
            (Taro as any).toast('手机号不正确')
        }
      }
  }

  async submit() {
      if (this.state.code.length !== 6) {
        (Taro as any).toast('验证码位数不够')
      } else {
        (Taro as any).loading()
        _api_bindPhone({
          code: (await Taro.login()).code,
          phone: this.state.phone,
          phone_code: this.state.code,
          nickname: this.props.userInfo.wx.nickName,
          avatar: this.props.userInfo.wx.avatarUrl
        }).then(({ data: { code, message, data }}) => {
            if (code === 200) {
              this.props.bindOK(data)
            } else {
              (Taro as any).toast(message)
            }
        })
      }
  }
}

export default BindPhone
