import { ComponentClass } from 'react'
import Taro, { Component, Config } from '@tarojs/taro'
import { View, Button, Text, Image } from '@tarojs/components'

import { connect } from '@tarojs/redux'
// import { add, minus, asyncAdd } from '../../actions/counter'
import { getUserInfo } from '../../actions/userinfo'

import { _api_login, _api_getInfo } from '../../apis/login'

import './index.scss'

import BindPhone from './components/BindPhone'

import index_banner from '../../static/index_banner.png'
import index_signup from '../../static/index_signup.png'
import index_upload from '../../static/index_upload.png'
import index_promise from '../../static/index-promise.png'

// #region 书写注意
//
// 目前 typescript 版本还无法在装饰器模式下将 Props 注入到 Taro.Component 中的 props 属性
// 需要显示声明 connect 的参数类型并通过 interface 的方式指定 Taro.Component 子类的 props
// 这样才能完成类型检查和 IDE 的自动提示
// 使用函数模式则无此限制
// ref: https://github.com/DefinitelyTyped/DefinitelyTyped/issues/20796
//
// #endregion

// type PageStateProps = {
//   counter: {
//     num: number
//   }
// }

type PageStateProps = {
  userInfo: {
    status: number,
    sign_start_time: string,
    sign_end_time: string,
    start_time: string,
    end_time: string,
    sign_up: boolean,
    week: number,
    nowNum: number,
    willNum: number
    wx: {
      nickName: string,
      avatarUrl: string
    }
  }
}

// type PageDispatchProps = {
//   add: () => void
//   dec: () => void
//   asyncAdd: () => any
// }

type PageDispatchProps = {
  getUserInfo: (e) => void
}

type PageOwnProps = {}
type PageState = {}
type IProps = PageStateProps & PageDispatchProps & PageOwnProps
interface Index {
  props: IProps;
}

// @connect(({ counter }) => ({
//   counter
// }), (dispatch) => ({
//   add () {
//     dispatch(add())
//   },
//   dec () {
//     dispatch(minus())
//   },
//   asyncAdd () {
//     dispatch(asyncAdd())
//   }
// }))
@connect(({ userInfo }) => ({
  userInfo
}), (dispatch) => ({
  getUserInfo (data) {
    dispatch(getUserInfo(data))
  }
}))
class Index extends Component<any, any> {
  config: Config = {
    navigationBarTitleText: '首页',
    enablePullDownRefresh: true
  }

  onShareAppMessage () {
    return {
      path: '/page/index/index'
    }
  }

  constructor(props) {
    super(props)
    this.state = {
      showBindPhone: false,
      showGetUserInfoButton: false
    }
  }
  
  async componentDidMount() { // 页面挂载，执行登陆
    (Taro as any).loading()
    this.getUserInfo()
  }

  componentDidShow () { }

  componentDidHide () { }

  render () {
    return (
      <View className='index'>
        <View className="time">{this.getDate(this.props.userInfo.start_time)}-{this.getDate(this.props.userInfo.end_time)}</View>
        {
          this.state.showBindPhone && <BindPhone bindOK={data => this.getInfo(data)} />
        }
        {
          this.state.showGetUserInfoButton && <Button className="getUserInfoButton" openType="getUserInfo" onGetUserInfo={({ detail }) => this.onGetUserInfo(detail)}>
            <Button>点击登陆实战营</Button>
          </Button>
        }
        <Image className="indexBanner" src={index_banner} />
        <View className="promiseNum">
          <Image className="promiseBg" src={index_promise} />
          <Text className="promiseText week">{ this.props.userInfo.week == 1 ? '第一周' : this.props.userInfo.week == 2 ? '第二周' : '第三周' }</Text>
          <Text className="promiseText nowNum">{ this.props.userInfo.nowNum }</Text>
          <Text className="promiseText willNum">{ this.props.userInfo.willNum }</Text>
        </View>
        <View className="indexItem">
          <View className="indexItemTitle">
            <Image src={index_signup} />
            <Text>实战营报名</Text>
          </View>
          <View className="indexItemDes">实战课堂，限时开启</View>
          <View className="indexItemDes">时间：{this.getDate(this.props.userInfo.sign_start_time)}-{this.getDate(this.props.userInfo.sign_end_time)}</View>
          <View className={'status'} >{this.props.userInfo.status === 1 ? '火爆开启' : '暂未开启'}</View>
          <Button onClick={this.enlist}>{this.props.userInfo.sign_up ? '我的报名' : '立即报名'}</Button>
        </View>
        <View className="indexItem">
          <View className="indexItemTitle">
              <Image src={index_upload} />
              <Text>图片上传</Text>
            </View>
            <View className="indexItemDes">上传截图，实战检验</View>
            <View className="indexItemDes">时间：{this.getDate(this.props.userInfo.start_time)}-{this.getDate(this.props.userInfo.end_time)}</View>
            <View className={'status'} >{this.props.userInfo.status === 3 ? '火爆开启' : '暂未开启'}</View>
            <Button onClick={this.upload}>上传图片</Button>
        </View>
      </View>
    )
  }

  getDate(dataTime: string) {
    return dataTime ? dataTime.split(' ')[0] : ''
  } 

  getUserInfo() { // api 获取用户信息
    Taro.getUserInfo().then(res => {
      Taro.hideLoading()
      this.onGetUserInfo(res) // 处理获取的用户信息
    }).catch(() => {
      Taro.hideLoading()
      Taro.setNavigationBarTitle({ title: '登陆' }) // 修改标题为 登陆
      this.setState({ showGetUserInfoButton: true }) // 显示登陆按钮
    })
  }

  onGetUserInfo(info) {
    if (Reflect.ownKeys(info).length > 1) { // 获取成功
      (Taro as any).loading()
      this.props.getUserInfo({ wx: info.userInfo })
      this.setState({ showGetUserInfoButton: false })
      Taro.startPullDownRefresh()
    } else { // 获取失败
      (Taro as any).toast('登陆失败')
    }
  }

  onPullDownRefresh() {
    if (this.props.userInfo.wx) {
      this.login()
    } else {
      Taro.stopPullDownRefresh()
    }
  }

  async login() { // 获取用户信息
    (Taro as any).loading()
    const openid = Taro.getStorageSync('openid')
    _api_login({
      openid,
      code: openid ? '' : (await Taro.login()).code,
      nickname: this.props.userInfo.wx.nickName,
      avatar: this.props.userInfo.wx.avatarUrl
    }).then(({ data: { code, message, data } }) => {
      if (code === 200) {
        this.getInfo(data)
      } else {
        Taro.hideLoading(); // 停止 loading
        Taro.stopPullDownRefresh(); // 停止下拉
        (Taro as any).toast(message) // 提示手机登陆
        this.setState({ showBindPhone: true }) // 显示手机登陆
      }
    })
  }

  async getInfo({ token }) { // 对获取的用户信息进行处理
    (Taro as any).token = token;
    const { data: { data }} = await _api_getInfo({})
    if (data.sign_up) {
      Taro.showTabBar(); // 显示 tabbar
    }
    Taro.hideLoading(); // 停止 loading
    Taro.stopPullDownRefresh(); // 停止下拉
    this.props.getUserInfo({ ...data, week: 1, nowNum: 18, willNum: 23 }); // 存储信息
    this.setState({ showBindPhone: false }); // 隐藏手机绑定
    Taro.setStorageSync('openid', data.openid); // 存储 openid
    Taro.setNavigationBarTitle({ title: '首页' }); // 设置标题为首页

    // Taro.navigateTo({ url: '/pages/army/army' })
  }

  enlist() { // 点击报名
    if (this.props.userInfo.sign_up) {
      Taro.navigateTo({ url: '/pages/enlist/enlist' })
    } else {
      if (this.props.userInfo.status === 1) {
        Taro.navigateTo({ url: '/pages/enlist/enlist' })
      } else {
        (Taro as any).toast('当前时间不可报名')
      }
    }
  }

  upload() { // 点击上传
    if (this.props.userInfo.status === 3) {
      if (this.props.userInfo.sign_up) {
        Taro.navigateTo({ url: '/pages/upload/upload' })
      } else {
        (Taro as any).toast('您未报名')
      }
    } else {
      (Taro as any).toast('当前时间不可上传')
    }
  }

}

export default Index as ComponentClass<PageOwnProps, PageState>
