import { ComponentClass } from 'react'
import Taro, { Component, Config } from '@tarojs/taro'
import { View, Button, Text, Image, OpenData } from '@tarojs/components'

import { connect } from '@tarojs/redux'
import { getUserInfo } from '../../actions/userinfo'

import { _api_rank } from '../../apis/rank'

import './rank.scss'

import img_rank from '../../static/rank.png'
import img_fir from '../../static/fir.png'
import img_sec from '../../static/sec.png'
import img_thi from '../../static/thi.png'
import img_normal from '../../static/normal.png'

type PageStateProps = {
  userInfo: {
    wx: {
      avatarUrl: string
    }
  }
}
type PageDispatchProps = {
  getUserInfo: (e) => void
}

type PageOwnProps = {}
type PageState = {}
type IProps = PageStateProps & PageDispatchProps & PageOwnProps
interface Index {
  props: IProps;
}

@connect(({ userInfo }) => ({
  userInfo
}), (dispatch) => ({
  getUserInfo (data) {
    dispatch(getUserInfo(data))
  }
}))
class Index extends Component<any, any> {
  config: Config = {
    navigationBarTitleText: '排行榜',
    onReachBottomDistance: 30,
    enablePullDownRefresh: true
  }

  constructor(props) {
    super(props)
    this.state = {
      person: {
        rank: 0,
        num: 0
      },
      list: [],
      requesting: false,
      page: 1
    }
  }

  componentDidShow() {
    Taro.startPullDownRefresh()
  }

  getAvatarDecorator(i) {
    switch (i) {
      case 0: {
        return <Image className="avatarDecorator" src={img_fir} />
      }
      case 1: {
        return <Image className="avatarDecorator" src={img_sec} />
      }
      case 2: {
        return <Image className="avatarDecorator" src={img_thi} />
      }
    }
    return <Image className="avatarDecorator" src={img_normal} />
  }

  render () {
    return (
      <View className='rank'>
        <Image className="rank_banner" src={img_rank} />
        <View className="person">
          <Text className="personRank">{this.state.person.rank || 0}</Text>
          <View className="avatarWrapper">
            <Image className="avatar" src={this.props.userInfo.wx.avatarUrl} />
            {
              this.getAvatarDecorator(this.state.person.rank - 1 || 0)
            }
          </View>
          <OpenData className="nickname" type='userNickName'/>
          <Text className="personNum">{this.state.person.num || 0}套</Text>
        </View>
        <View className="list">
          <View className="listTitle">实时更新</View>
          {
            this.state.list.map((e, i) => (<View className={'listItem' + ' ' + (i > 2 ? 'normal' : '')} key={e.id}>
              <Text className="personRank">{i + 1}</Text>
              <View className="avatarWrapper">
                <Image className="avatar" src={e.avatar} />
                {
                  this.getAvatarDecorator(i)
                }
              </View>
              <Text className="nickname">{e.nickname}</Text>
              <Text className="personNum">{e.num}套</Text>
            </View>))
          }
        </View>
        <View className="loading">{ this.state.page > 0 ? '加载中...' : '没有更多了' }</View>
      </View>
    )
  }

  onPullDownRefresh() {
    this.setState({ page: 1 }, () => this.request(true))
  }

  onReachBottom() {
    this.request(false)
  }

  request(loading: boolean) {
    if (!this.state.requesting && this.state.page > 0) {
      this.setState({ requesting: true })
      if (loading) {
        (Taro as any).loading()
      }
      _api_rank({ page: this.state.page }).then(({ data : { data: { size, list, rank, num } }}) => {
        this.setState({
          person: { rank, num },
          requesting: false,
          page: list.length < size ? 0 : this.state.page + 1,
          list: loading ? list : [...this.state.list, ...list]
        })
        Taro.hideLoading()
        Taro.stopPullDownRefresh()
      })
    }
  }

}

export default Index as ComponentClass<PageOwnProps, PageState>
