import { ComponentClass } from 'react'
import Taro, { Component, Config } from '@tarojs/taro'
import { View, Image, Text } from '@tarojs/components'

import { connect } from '@tarojs/redux'
import { getUserInfo } from '../../actions/userinfo'

import { _api_login } from '../../apis/login'

import './upload.scss'

import Counter from './components/Counter'

import { _api_upload } from '../../apis/upload'

type PageStateProps = {
  userInfo: object
}
type PageDispatchProps = {
  getUserInfo: (e) => void
}

type PageOwnProps = {}
type PageState = {}
type IProps = PageStateProps & PageDispatchProps & PageOwnProps
interface Index {
  props: IProps;
}

@connect(({ userInfo }) => ({
  userInfo
}), (dispatch) => ({
  getUserInfo (data) {
    dispatch(getUserInfo(data))
  }
}))
class Index extends Component<any, any> {
  config: Config = {
    navigationBarTitleText: '上传图片'
  }

  constructor(props) {
    super(props)
    this.state = {
        hard: 0,
        simple: 0,
        old: 0,
        imgList: []
    }
  }
  
  async componentDidMount() { }

  componentDidShow () { }

  componentDidHide () { }

  render () {
    return (
      <View className='upload'>
        <View className="app-item">
          <Text>精装版</Text> 
          <Counter value={this.state.hard} onChange={hard => this.setState({ hard })} />
        </View>
        <View className="app-item">
          <Text>简约版</Text>
          <Counter value={this.state.simple} onChange={simple => this.setState({ simple })} />
        </View>
        <View className="app-item">
          <Text>老人版</Text> 
          <Counter value={this.state.old} onChange={old => this.setState({ old })} />
        </View>
        <View className="choose" onClick={this.chooseImage}>
          {
            this.state.imgList.length > 0 ?
              <View className="preview">
                <Image src={this.state.imgList[0]} onClick={this.preview} />
              </View> :
              <View className="preview">
                <View className="at-icon at-icon-camera"></View>
                <Text>图片上传</Text>
              </View>
            }
        </View>
        <View className="submit" onClick={this.submit}>提交</View>
      </View>
    )
  }

  chooseImage() {
    Taro.chooseImage({
      count: 1,
      sizeType: ['original'], //可以指定是原图还是压缩图，默认二者都有
      sourceType: ['album'], //从相册选择
      success: ({ tempFilePaths: imgList }) => {
        this.setState({ imgList })
      }
    });
  }

  preview(e) {
    e.stopPropagation()
    Taro.previewImage({ urls: this.state.imgList })
  }

  submit() {
    if (this.state.hard + this.state.simple + this.state.old) {
      if (this.state.imgList.length) {
        (Taro as any).loading();
        const task = _api_upload({
          name: 'img',
          formData: this.state,
          filePath: this.state.imgList[0],
          success: ({ data: res }) => {
            typeof res === 'string' ? res = JSON.parse(res) : ''
            const { code, message, data } = res
            if (code === 200) {
              this.setState({
                hard: 0, 
                simple: 0,
                old: 0,
                imgList: []
              })
              console.log(data)
              Taro.showModal({
                title: '提示',
                confirmText: '继续上传',
                cancelText: '查看排名',
                content: '上传成功',
                success: ({ cancel }) => {
                  if (cancel) {
                    Taro.switchTab({ url: '/pages/rank/rank' })
                  }
                }
              })
            } else {
              Taro.showModal({
                title: '上传失败',
                showCancel: false,
                content: message
              })
            }
          },
          complete: () => {
            Taro.hideLoading()
          }
        })
        console.log(task)
        task.progress(({ progress }) => console.log(progress))
      } else {
        (Taro as any).toast('图片不能为空')
      }
    } else {
      (Taro as any).toast('数量不能为空')
    }
  }

}

export default Index as ComponentClass<PageOwnProps, PageState>
